@@ -36,7 +36,7 @@
  */
 
 #include "ftp_locl.h"
-RCSID("$Id: cmds.c,v 1.33 1998/09/01 14:56:31 joda Exp $");
+RCSID("$Id: cmds.c,v 1.47 2005/04/18 07:45:12 lha Exp $");
 
 typedef void (*sighand)(int);
 
@@ -81,7 +81,7 @@ void
 setpeer(int argc, char **argv)
 {
 	char *host;
-	short port;
+	u_short port;
 	struct servent *sp;
 
 	if (connected) {
@@ -102,14 +102,23 @@ setpeer(int argc, char **argv)
 		errx(1, "You bastard. You removed ftp/tcp from services");
 	port = sp->s_port;
 	if (argc > 2) {
-		port = atoi(argv[2]);
-		if (port <= 0) {
-			printf("%s: bad port number-- %s\n", argv[1], argv[2]);
-			printf ("usage: %s host-name [port]\n", argv[0]);
-			code = -1;
-			return;
+		sp = getservbyname(argv[2], "tcp");
+		if (sp != NULL) {
+			port = sp->s_port;
+		} else {
+			char *ep;
+
+			port = strtol(argv[2], &ep, 0);
+			if (argv[2] == ep) {
+				printf("%s: bad port number-- %s\n",
+				       argv[1], argv[2]);
+				printf ("usage: %s host-name [port]\n",
+					argv[0]);
+				code = -1;
+				return;
+			}
+			port = htons(port);
 		}
-		port = htons(port);
 	}
 	host = hookup(argv[1], port);
 	if (host) {
@@ -119,21 +128,21 @@ setpeer(int argc, char **argv)
 		/*
 		 * Set up defaults for FTP.
 		 */
-		strcpy_truncate(typename, "ascii", sizeof(typename));
+		strlcpy(typename, "ascii", sizeof(typename));
 		type = TYPE_A;
 		curtype = TYPE_A;
-		strcpy_truncate(formname, "non-print", sizeof(formname));
+		strlcpy(formname, "non-print", sizeof(formname));
 		form = FORM_N;
-		strcpy_truncate(modename, "stream", sizeof(modename));
+		strlcpy(modename, "stream", sizeof(modename));
 		mode = MODE_S;
-		strcpy_truncate(structname, "file", sizeof(structname));
+		strlcpy(structname, "file", sizeof(structname));
 		stru = STRU_F;
-		strcpy_truncate(bytename, "8", sizeof(bytename));
+		strlcpy(bytename, "8", sizeof(bytename));
 		bytesize = 8;
 		if (autologin)
 			login(argv[1]);
 
-#if (defined(unix) || defined(__unix__) || defined(__unix) || defined(_AIX) || defined(_CRAY)) && NBBY == 8
+#if (defined(unix) || defined(__unix__) || defined(__unix) || defined(_AIX) || defined(_CRAY) || defined(__NetBSD__) || defined(__APPLE__)) && NBBY == 8
 /*
  * this ifdef is to keep someone form "porting" this to an incompatible
  * system and not checking this out. This way they have to think about it.
@@ -141,22 +150,23 @@ setpeer(int argc, char **argv)
 		overbose = verbose;
 		if (debug == 0)
 			verbose = -1;
-		if (command("SYST") == COMPLETE && overbose) {
-			char *cp, c;
-			cp = strchr(reply_string+4, ' ');
+		if (command("SYST") == COMPLETE && overbose && strlen(reply_string) > 4) {
+			char *cp, *p;
+
+			cp = strdup(reply_string + 4);
 			if (cp == NULL)
-				cp = strchr(reply_string+4, '\r');
-			if (cp) {
-				if (cp[-1] == '.')
-					cp--;
-				c = *cp;
-				*cp = '\0';
+			    errx(1, "strdup: out of memory");
+			p = strchr(cp, ' ');
+			if (p == NULL)
+				p = strchr(cp, '\r');
+			if (p) {
+				if (p[-1] == '.')
+					p--;
+				*p = '\0';
 			}
 
-			printf("Remote system type is %s.\n",
-				reply_string+4);
-			if (cp)
-				*cp = c;
+			printf("Remote system type is %s.\n", cp);
+			free(cp);
 		}
 		if (!strncmp(reply_string, "215 UNIX Type: L8", 17)) {
 			if (proxy)
@@ -170,7 +180,7 @@ setpeer(int argc, char **argv)
 			 * for text files unless changed by the user.
 			 */
 			type = 0;
-			strcpy_truncate(typename, "binary", sizeof(typename));
+			strlcpy(typename, "binary", sizeof(typename));
 			if (overbose)
 			    printf("Using %s mode to transfer files.\n",
 				typename);
@@ -243,7 +253,7 @@ settype(int argc, char **argv)
 	else
 		comret = command("TYPE %s", p->t_mode);
 	if (comret == COMPLETE) {
-		strcpy_truncate(typename, p->t_name, sizeof(typename));
+		strlcpy(typename, p->t_name, sizeof(typename));
 		curtype = type = p->t_type;
 	}
 }
@@ -403,7 +413,8 @@ usage:
 		argv[2] = domap(argv[2]);
 	}
 	sendrequest(cmd, argv[1], argv[2],
-	    argv[1] != oldargv1 || argv[2] != oldargv2);
+		    curtype == TYPE_I ? "rb" : "r",
+		    argv[1] != oldargv1 || argv[2] != oldargv2);
 }
 
 /* ARGSUSED */
@@ -434,7 +445,7 @@ void
 mput(int argc, char **argv)
 {
     int i;
-    RETSIGTYPE (*oldintr)();
+    RETSIGTYPE (*oldintr)(int);
     int ointer;
     char *tp;
 
@@ -458,14 +469,14 @@ mput(int argc, char **argv)
 	    if (mflag && confirm(argv[0], cp)) {
 		tp = cp;
 		if (mcase) {
-		    while (*tp && !islower(*tp)) {
+		    while (*tp && !islower((unsigned char)*tp)) {
 			tp++;
 		    }
 		    if (!*tp) {
 			tp = cp;
 			tp2 = tmpbuf;
 			while ((*tp2 = *tp) != '\0') {
-			    if (isupper(*tp2)) {
+			    if (isupper((unsigned char)*tp2)) {
 				*tp2 = 'a' + *tp2 - 'A';
 			    }
 			    tp++;
@@ -481,7 +492,9 @@ mput(int argc, char **argv)
 		    tp = domap(tp);
 		}
 		sendrequest((sunique) ? "STOU" : "STOR",
-			    cp, tp, cp != tp || !interactive);
+			    cp, tp,
+			    curtype == TYPE_I ? "rb" : "r",
+			    cp != tp || !interactive);
 		if (!mflag && fromatty) {
 		    ointer = interactive;
 		    interactive = 1;
@@ -506,7 +519,9 @@ mput(int argc, char **argv)
 		tp = (ntflag) ? dotrans(argv[i]) : argv[i];
 		tp = (mapflag) ? domap(tp) : tp;
 		sendrequest((sunique) ? "STOU" : "STOR",
-			    argv[i], tp, tp != argv[i] || !interactive);
+			    argv[i],
+			    curtype == TYPE_I ? "rb" : "r",
+			    tp, tp != argv[i] || !interactive);
 		if (!mflag && fromatty) {
 		    ointer = interactive;
 		    interactive = 1;
@@ -531,7 +546,9 @@ mput(int argc, char **argv)
 		tp = (ntflag) ? dotrans(*cpp) : *cpp;
 		tp = (mapflag) ? domap(tp) : tp;
 		sendrequest((sunique) ? "STOU" : "STOR",
-			    *cpp, tp, *cpp != tp || !interactive);
+			    *cpp, tp,
+			    curtype == TYPE_I ? "rb" : "r",
+			    *cpp != tp || !interactive);
 		if (!mflag && fromatty) {
 		    ointer = interactive;
 		    interactive = 1;
@@ -551,13 +568,27 @@ mput(int argc, char **argv)
 void
 reget(int argc, char **argv)
 {
-    getit(argc, argv, 1, "r+w");
+    getit(argc, argv, 1, curtype == TYPE_I ? "r+wb" : "r+w");
 }
 
 void
 get(int argc, char **argv)
 {
-    getit(argc, argv, 0, restart_point ? "r+w" : "w" );
+    char *mode;
+
+    if (restart_point) {
+	if (curtype == TYPE_I)
+	    mode = "r+wb";
+	else
+	    mode = "r+w";
+    } else {
+	if (curtype == TYPE_I)
+	    mode = "wb";
+	else
+	    mode = "w";
+    }
+
+    getit(argc, argv, 0, mode);
 }
 
 /*
@@ -591,14 +622,14 @@ getit(int argc, char **argv, int restartit, char *mode)
 	if (loc && mcase) {
 		char *tp = argv[1], *tp2, tmpbuf[MaxPathLen];
 
-		while (*tp && !islower(*tp)) {
+		while (*tp && !islower((unsigned char)*tp)) {
 			tp++;
 		}
 		if (!*tp) {
 			tp = argv[2];
 			tp2 = tmpbuf;
 			while ((*tp2 = *tp) != '\0') {
-				if (isupper(*tp2)) {
+				if (isupper((unsigned char)*tp2)) {
 					*tp2 = 'a' + *tp2 - 'A';
 				}
 				tp++;
@@ -622,41 +653,46 @@ getit(int argc, char **argv, int restartit, char *mode)
 				return (0);
 			}
 			restart_point = stbuf.st_size;
-		} else {
-			if (ret == 0) {
-				int overbose;
+		} else if (ret == 0) {
+			int overbose;
+			int cmdret;
+			int yy, mo, day, hour, min, sec;
+			struct tm *tm;
+			time_t mtime = stbuf.st_mtime;
 
-				overbose = verbose;
-				if (debug == 0)
-					verbose = -1;
-				if (command("MDTM %s", argv[1]) == COMPLETE) {
-					int yy, mo, day, hour, min, sec;
-					struct tm *tm;
-					verbose = overbose;
-					sscanf(reply_string,
-					    "%*s %04d%02d%02d%02d%02d%02d",
-					    &yy, &mo, &day, &hour, &min, &sec);
-					tm = gmtime(&stbuf.st_mtime);
-					tm->tm_mon++;
-					if (tm->tm_year > yy%100)
-						return (1);
-					if ((tm->tm_year == yy%100 && 
-					    tm->tm_mon > mo) ||
-					   (tm->tm_mon == mo && 
-					    tm->tm_mday > day) ||
-					   (tm->tm_mday == day && 
-					    tm->tm_hour > hour) ||
-					   (tm->tm_hour == hour && 
-					    tm->tm_min > min) ||
-					   (tm->tm_min == min && 
-					    tm->tm_sec > sec))
-						return (1);
-				} else {
-					printf("%s\n", reply_string);
-					verbose = overbose;
-					return (0);
-				}
+			overbose = verbose;
+			if (debug == 0)
+				verbose = -1;
+			cmdret = command("MDTM %s", argv[1]);
+			verbose = overbose;
+			if (cmdret != COMPLETE) {
+				printf("%s\n", reply_string);
+				return (0);
 			}
+			if (sscanf(reply_string,
+				   "%*s %04d%02d%02d%02d%02d%02d",
+				   &yy, &mo, &day, &hour, &min, &sec)
+			    != 6) {
+				printf ("bad MDTM result\n");
+				return (0);
+			}
+
+			tm = gmtime(&mtime);
+			tm->tm_mon++;
+			tm->tm_year += 1900;
+
+			if ((tm->tm_year > yy) ||
+			    (tm->tm_year == yy && 
+			     tm->tm_mon > mo) ||
+			    (tm->tm_mon == mo && 
+			     tm->tm_mday > day) ||
+			    (tm->tm_mday == day && 
+			     tm->tm_hour > hour) ||
+			    (tm->tm_hour == hour && 
+			     tm->tm_min > min) ||
+			    (tm->tm_min == min && 
+			     tm->tm_sec > sec))
+				return (1);
 		}
 	}
 
@@ -701,8 +737,8 @@ mget(int argc, char **argv)
 		if (mflag && confirm(argv[0], cp)) {
 			tp = cp;
 			if (mcase) {
-				for (tp2 = tmpbuf; (ch = *tp++);)
-					*tp2++ = isupper(ch) ? tolower(ch) : ch;
+				for (tp2 = tmpbuf;(ch = (unsigned char)*tp++);)
+					*tp2++ = tolower(ch);
 				*tp2 = '\0';
 				tp = tmpbuf;
 			}
@@ -712,8 +748,9 @@ mget(int argc, char **argv)
 			if (mapflag) {
 				tp = domap(tp);
 			}
-			recvrequest("RETR", tp, cp, "w",
-			    tp != cp || !interactive, 0);
+			recvrequest("RETR", tp, cp,
+				    curtype == TYPE_I ? "wb" : "w",
+				    tp != cp || !interactive, 0);
 			if (!mflag && fromatty) {
 				ointer = interactive;
 				interactive = 1;
@@ -759,7 +796,7 @@ remglob(char **argv, int doswitch)
     }
     if (ftemp == NULL) {
 	int fd;
-	strcpy_truncate(temp, _PATH_TMP_XXX, sizeof(temp));
+	strlcpy(temp, _PATH_TMP_XXX, sizeof(temp));
 	fd = mkstemp(temp);
 	if(fd < 0){
 	    warn("unable to create temporary file %s", temp);
@@ -1198,7 +1235,7 @@ void
 shell(int argc, char **argv)
 {
 	pid_t pid;
-	RETSIGTYPE (*old1)(), (*old2)();
+	RETSIGTYPE (*old1)(int), (*old2)(int);
 	char shellnam[40], *shell, *namep; 
 	int status;
 
@@ -1267,7 +1304,7 @@ user(int argc, char **argv)
 	n = command("USER %s", argv[1]);
 	if (n == CONTINUE) {
 	    if (argc < 3 ) {
-		des_read_pw_string (tmp,
+		UI_UTIL_read_pw_string (tmp,
 				    sizeof(tmp),
 				    "Password: ", 0);
 		argv[2] = tmp;
@@ -1394,11 +1431,11 @@ quote1(char *initial, int argc, char **argv)
     int i;
     char buf[BUFSIZ];		/* must be >= sizeof(line) */
 
-    strcpy_truncate(buf, initial, sizeof(buf));
+    strlcpy(buf, initial, sizeof(buf));
     for(i = 1; i < argc; i++) {
 	if(i > 1)
-	    strcat_truncate(buf, " ", sizeof(buf));
-	strcat_truncate(buf, argv[i], sizeof(buf));
+	    strlcat(buf, " ", sizeof(buf));
+	strlcat(buf, argv[i], sizeof(buf));
     }
     if (command("%s", buf) == PRELIM) {
 	while (getreply(0) == PRELIM)
@@ -1550,15 +1587,15 @@ account(int argc, char **argv)
 	if (argc > 1) {
 		++argv;
 		--argc;
-		strcpy_truncate (acct, *argv, sizeof(acct));
+		strlcpy (acct, *argv, sizeof(acct));
 		while (argc > 1) {
 			--argc;
 			++argv;
-			strcat_truncate(acct, *argv, sizeof(acct));
+			strlcat(acct, *argv, sizeof(acct));
 		}
 	}
 	else {
-	    des_read_pw_string(acct, sizeof(acct), "Account:", 0);
+	    UI_UTIL_read_pw_string(acct, sizeof(acct), "Account:", 0);
 	}
 	command("ACCT %s", acct);
 }
@@ -1586,7 +1623,7 @@ void
 doproxy(int argc, char **argv)
 {
 	struct cmd *c;
-	RETSIGTYPE (*oldintr)();
+	RETSIGTYPE (*oldintr)(int);
 
 	if (argc < 2 && !another(&argc, &argv, "command")) {
 		printf("usage: %s command\n", argv[0]);
@@ -1666,12 +1703,12 @@ setntrans(int argc, char **argv)
 	}
 	ntflag++;
 	code = ntflag;
-	strcpy_truncate (ntin, argv[1], 17);
+	strlcpy (ntin, argv[1], 17);
 	if (argc == 2) {
 		ntout[0] = '\0';
 		return;
 	}
-	strcpy_truncate (ntout, argv[2], 17);
+	strlcpy (ntout, argv[2], 17);
 }
 
 char *
@@ -1728,10 +1765,10 @@ setnmap(int argc, char **argv)
 		cp = strchr(altarg, ' ');
 	}
 	*cp = '\0';
-	strcpy_truncate(mapin, altarg, MaxPathLen);
+	strlcpy(mapin, altarg, MaxPathLen);
 	while (*++cp == ' ')
 		continue;
-	strcpy_truncate(mapout, cp, MaxPathLen);
+	strlcpy(mapout, cp, MaxPathLen);
 }
 
 char *
@@ -1795,7 +1832,7 @@ domap(char *name)
 				break;
 			case '[':
 LOOP:
-				if (*++cp2 == '$' && isdigit(*(cp2+1))) { 
+				if (*++cp2 == '$' && isdigit((unsigned char)*(cp2+1))) { 
 					if (*++cp2 == '0') {
 						char *cp3 = name;
 
@@ -1820,7 +1857,7 @@ LOOP:
 							cp2++;
 						}
 						else if (*cp2 == '$' &&
-   						        isdigit(*(cp2+1))) {
+   						        isdigit((unsigned char)*(cp2+1))) {
 							if (*++cp2 == '0') {
 							   char *cp3 = name;
 
@@ -1872,7 +1909,7 @@ LOOP:
 				}
 				break;
 			case '$':
-				if (isdigit(*(cp2 + 1))) {
+				if (isdigit((unsigned char)*(cp2 + 1))) {
 					if (*++cp2 == '0') {
 						char *cp3 = name;
 
@@ -1983,7 +2020,7 @@ macdef(int argc, char **argv)
 	if (interactive) {
 		printf("Enter macro line by line, terminating it with a null line\n");
 	}
-	strcpy_truncate(macros[macnum].mac_name,
+	strlcpy(macros[macnum].mac_name,
 			argv[1],
 			sizeof(macros[macnum].mac_name));
 	if (macnum == 0) {
@@ -2085,7 +2122,7 @@ void
 newer(int argc, char **argv)
 {
 
-	if (getit(argc, argv, -1, "w"))
+	if (getit(argc, argv, -1, curtype == TYPE_I ? "wb" : "w"))
 		printf("Local file \"%s\" is newer than remote file \"%s\"\n",
 			argv[2], argv[1]);
 }
