@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2001, 2003 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-2004 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -32,7 +32,7 @@
  */
 
 #include "push_locl.h"
-RCSID("$Id: push.c,v 1.47 2003/04/04 02:10:17 assar Exp $");
+RCSID("$Id: push.c,v 1.49 2005/04/19 18:00:17 lha Exp $");
 
 #ifdef KRB4
 static int use_v4 = -1;
@@ -214,9 +214,10 @@ doit(int s,
     int ret;
     char out_buf[PUSH_BUFSIZ];
     int out_len = 0;
-    char in_buf[PUSH_BUFSIZ + 1];	/* sentinel */
+    char *in_buf;
+    size_t in_buf_size;
     size_t in_len = 0;
-    char *in_ptr = in_buf;
+    char *in_ptr;
     pop_state state = INIT;
     unsigned count, bytes;
     unsigned asked_for = 0, retrieved = 0, asked_deleted = 0, deleted = 0;
@@ -231,6 +232,10 @@ doit(int s,
     int i;
     char *tmp = NULL;
 
+    in_buf = emalloc(PUSH_BUFSIZ + 1);
+    in_ptr = in_buf;
+    in_buf_size = PUSH_BUFSIZ;
+
     if (do_from) {
 	char *tmp2;
 
@@ -263,11 +268,13 @@ doit(int s,
     now = time(NULL);
     from_line_length = snprintf (from_line, sizeof(from_line),
 				 "From %s %s", "push", ctime(&now));
+    if (from_line_length < 0 || from_line_length > sizeof(from_line))
+	errx (1, "snprintf failed");
 
     out_len = snprintf (out_buf, sizeof(out_buf),
 			"USER %s\r\nPASS hej\r\nSTAT\r\n",
 			user);
-    if (out_len < 0)
+    if (out_len < 0 || out_len > sizeof(out_buf))
 	errx (1, "snprintf failed");
     if (net_write (s, out_buf, out_len) != out_len)
 	err (1, "write");
@@ -310,7 +317,14 @@ doit(int s,
 	    size_t rem;
 	    int blank_line = 0;
 	    
-	    ret = read (s, in_ptr, sizeof(in_buf) - in_len - 1);
+	    if(in_len >= in_buf_size) {
+		char *tmp = erealloc(in_buf, in_buf_size + PUSH_BUFSIZ + 1);
+		in_ptr = tmp + (in_ptr - in_buf);
+		in_buf = tmp;
+		in_buf_size += PUSH_BUFSIZ;
+	    }
+
+	    ret = read (s, in_ptr, in_buf_size - in_len);
 	    if (ret < 0)
 		err (1, "read");
 	    else if (ret == 0)
@@ -478,7 +492,7 @@ doit(int s,
 	    else if(state == DELE)
 		out_len = snprintf (out_buf, sizeof(out_buf),
 				    "DELE %u\r\n", ++asked_deleted);
-	    if (out_len < 0)
+	    if (out_len < 0 || out_len > sizeof(out_buf))
 		errx (1, "snprintf failed");
 	    if (net_write (s, out_buf, out_len) != out_len)
 		err (1, "write");
