@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 - 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-2004 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -37,22 +32,56 @@
  */
 
 #include "rsh_locl.h"
-RCSID("$Id: common.c,v 1.11 1999/10/26 04:14:36 assar Exp $");
+RCSID("$Id: common.c,v 1.17 2004/09/21 11:32:54 joda Exp $");
+
+#if defined(KRB4) || defined(KRB5)
+
+#ifdef KRB5
+int key_usage = 1026;
+
+void *ivec_in[2];
+void *ivec_out[2];
+
+void
+init_ivecs(int client, int have_errsock)
+{
+    size_t blocksize;
+
+    krb5_crypto_getblocksize(context, crypto, &blocksize);
+
+    ivec_in[0] = malloc(blocksize);
+    memset(ivec_in[0], client, blocksize);
+
+    if(have_errsock) {
+	ivec_in[1] = malloc(blocksize);
+	memset(ivec_in[1], 2 | client, blocksize);
+    } else
+	ivec_in[1] = ivec_in[0];
+
+    ivec_out[0] = malloc(blocksize);
+    memset(ivec_out[0], !client, blocksize);
+
+    if(have_errsock) {
+	ivec_out[1] = malloc(blocksize);
+	memset(ivec_out[1], 2 | !client, blocksize);
+    } else
+	ivec_out[1] = ivec_out[0];
+}
+#endif
+
 
 ssize_t
-do_read (int fd,
-	 void *buf,
-	 size_t sz)
+do_read (int fd, void *buf, size_t sz, void *ivec)
 {
-    int ret;
-
     if (do_encrypt) {
 #ifdef KRB4
 	if (auth_method == AUTH_KRB4) {
 	    return des_enc_read (fd, buf, sz, schedule, &iv);
 	} else
 #endif /* KRB4 */
+#ifdef KRB5
         if(auth_method == AUTH_KRB5) {
+	    krb5_error_code ret;
 	    u_int32_t len, outer_len;
 	    int status;
 	    krb5_data data;
@@ -64,7 +93,11 @@ do_read (int fd,
 	    len = ntohl(len);
 	    if (len > sz)
 		abort ();
-	    outer_len = krb5_get_wrapped_length (context, crypto, len);
+	    /* ivec will be non null for protocol version 2 */
+	    if(ivec != NULL)
+		outer_len = krb5_get_wrapped_length (context, crypto, len + 4);
+	    else
+		outer_len = krb5_get_wrapped_length (context, crypto, len);
 	    edata = malloc (outer_len);
 	    if (edata == NULL)
 		errx (1, "malloc: cannot allocate %u bytes", outer_len);
@@ -72,24 +105,33 @@ do_read (int fd,
 	    if (ret <= 0)
 		return ret;
 
-	    status = krb5_decrypt(context, crypto, KRB5_KU_OTHER_ENCRYPTED, 
-				  edata, outer_len, &data);
+	    status = krb5_decrypt_ivec(context, crypto, key_usage, 
+				       edata, outer_len, &data, ivec);
 	    free (edata);
 	    
 	    if (status)
-		errx (1, "%s", krb5_get_err_text (context, status));
-	    memcpy (buf, data.data, len);
+		krb5_err (context, 1, status, "decrypting data");
+	    if(ivec != NULL) {
+		unsigned long l;
+		if(data.length < len + 4)
+		    errx (1, "data received is too short");
+		_krb5_get_int(data.data, &l, 4);
+		if(l != len)
+		    errx (1, "inconsistency in received data");
+		memcpy (buf, (unsigned char *)data.data+4, len);
+	    } else
+		memcpy (buf, data.data, len);
 	    krb5_data_free (&data);
 	    return len;
-	} else {
+	} else
+#endif /* KRB5 */
 	    abort ();
-	}
     } else
 	return read (fd, buf, sz);
 }
 
 ssize_t
-do_write (int fd, void *buf, size_t sz)
+do_write (int fd, void *buf, size_t sz, void *ivec)
 {
     if (do_encrypt) {
 #ifdef KRB4
@@ -97,23 +139,31 @@ do_write (int fd, void *buf, size_t sz)
 	    return des_enc_write (fd, buf, sz, schedule, &iv);
 	} else
 #endif /* KRB4 */
+#ifdef KRB5
 	if(auth_method == AUTH_KRB5) {
 	    krb5_error_code status;
 	    krb5_data data;
-	    u_int32_t len;
+	    unsigned char len[4];
 	    int ret;
 
-	    status = krb5_encrypt(context, crypto, KRB5_KU_OTHER_ENCRYPTED,
-				  buf, sz, &data);
-	    
+	    _krb5_put_int(len, sz, 4);
+	    if(ivec != NULL) {
+		unsigned char *tmp = malloc(sz + 4);
+		if(tmp == NULL)
+		    err(1, "malloc");
+		_krb5_put_int(tmp, sz, 4);
+		memcpy(tmp + 4, buf, sz);
+		status = krb5_encrypt_ivec(context, crypto, key_usage,
+					   tmp, sz + 4, &data, ivec);
+		free(tmp);
+	    } else
+		status = krb5_encrypt_ivec(context, crypto, key_usage,
+					   buf, sz, &data, ivec);
+
 	    if (status)
-		errx (1, "%s", krb5_get_err_text(context, status));
+		krb5_err(context, 1, status, "encrypting data");
 
-	    assert (krb5_get_wrapped_length (context, crypto,
-					     sz) == data.length);
-
-	    len = htonl(sz);
-	    ret = krb5_net_write (context, &fd, &len, 4);
+	    ret = krb5_net_write (context, &fd, len, 4);
 	    if (ret != 4)
 		return ret;
 	    ret = krb5_net_write (context, &fd, data.data, data.length);
@@ -121,9 +171,10 @@ do_write (int fd, void *buf, size_t sz)
 		return ret;
 	    free (data.data);
 	    return sz;
-	} else {
+	} else
+#endif /* KRB5 */
 	    abort();
-	}
     } else
 	return write (fd, buf, sz);
 }
+#endif /* KRB4 || KRB5 */
