@@ -32,7 +32,7 @@
  */
 
 #include "rsh_locl.h"
-RCSID("$Id: rsh.c,v 1.78 2005/04/24 20:28:59 lha Exp $");
+RCSID("$Id: rsh.c,v 1.78.2.1 2005/12/28 18:00:05 lha Exp $");
 
 enum auth_method auth_method;
 #if defined(KRB4) || defined(KRB5)
@@ -325,6 +325,10 @@ send_krb5_auth(int s,
 				  do_encrypt ? "-x " : "",
 				  cmd,
 				  remote_user);
+    if (cksum_data.length == -1) {
+	warnx ("%s: failed to allocate command", hostname);
+	return 1;
+    }
 
     ap_opts = 0;
 
