@@ -38,12 +38,7 @@ static char *copyright[] = {
 };
 
 #include "telnet_locl.h"
-RCSID("$Id: main.c,v 1.29 1999/09/16 20:41:36 assar Exp $");
-
-/* These values need to be the same as defined in libtelnet/kerberos5.c */
-/* Either define them in both places, or put in some common header file. */
-#define OPTS_FORWARD_CREDS	0x00000002
-#define OPTS_FORWARDABLE_CREDS	0x00000001
+RCSID("$Id: main.c,v 1.40 2004/04/02 21:28:48 lha Exp $");
 
 #if KRB5
 #define FORWARD
@@ -91,37 +86,65 @@ usage(void)
 
 
 #ifdef	FORWARD
-extern int forward_flags;
-static int default_forward=0;
+int forward_option = 0; /* forward flags set from command line */
 #endif	/* FORWARD */
+void
+set_forward_options(void)
+{
+#ifdef FORWARD
+	switch(forward_option) {
+	case 'f':
+		kerberos5_set_forward(1);
+		kerberos5_set_forwardable(0);
+		break;
+	case 'F':
+		kerberos5_set_forward(1);
+		kerberos5_set_forwardable(1);
+		break;
+	case 'G':
+		kerberos5_set_forward(0);
+		kerberos5_set_forwardable(0);
+		break;
+	default:
+		break;
+	}
+#endif
+}
 
 #ifdef KRB5
-/* XXX ugly hack to setup dns-proxy stuff */
 #define Authenticator asn1_Authenticator
 #include <krb5.h>
 static void
 krb5_init(void)
 {
     krb5_context context;
-    krb5_init_context(&context);
+    krb5_error_code ret;
+    krb5_boolean ret_val;
 
-#if defined(AUTHENTICATION) && defined(KRB5) && defined(FORWARD)
-    if (krb5_config_get_bool (context, NULL,
-         "libdefaults", "forward", NULL)) {
-           forward_flags |= OPTS_FORWARD_CREDS;
-           default_forward=1;
-    }
-    if (krb5_config_get_bool (context, NULL,
-         "libdefaults", "forwardable", NULL)) {
-           forward_flags |= OPTS_FORWARDABLE_CREDS;
-           default_forward=1;
-    }
+    ret = krb5_init_context(&context);
+    if (ret)
+	return;
+
+#if defined(AUTHENTICATION) && defined(FORWARD)
+    krb5_appdefault_boolean(context, NULL,
+			    NULL, "forward",
+			    0, &ret_val);
+    if (ret_val)
+	    kerberos5_set_forward(1);
+    krb5_appdefault_boolean(context, NULL,
+			    NULL, "forwardable",
+			    0, &ret_val);
+    if (ret_val)
+	    kerberos5_set_forwardable(1);
 #endif
 #ifdef  ENCRYPTION
-    if (krb5_config_get_bool (context, NULL,
-        "libdefaults", "encrypt", NULL)) {
+    krb5_appdefault_boolean(context, NULL, 
+			    NULL, "encrypt",
+			    0, &ret_val);
+    if (ret_val) {
           encrypt_auto(1);
           decrypt_auto(1); 
+	  wantencryption = 1;
           EncryptVerbose(1);
         }
 #endif
@@ -130,12 +153,19 @@ krb5_init(void)
 }
 #endif
 
+#if defined(AUTHENTICATION) && defined(KRB4)
+extern char *dest_realm, dst_realm_buf[];
+extern int dst_realm_sz;
+#endif
+
 int
 main(int argc, char **argv)
 {
 	int ch;
 	char *user;
 
+	setprogname(argv[0]);
+
 #ifdef KRB5
 	krb5_init();
 #endif
@@ -160,8 +190,13 @@ main(int argc, char **argv)
 	 */
 	autologin = -1;
 
+	if (argc == 2 && strcmp(argv[1], "--version") == 0) {
+	    print_version(NULL);
+	    exit(0);
+	}
+
 	while((ch = getopt(argc, argv,
-			   "78DEKLS:X:abcde:fFk:l:n:rxG")) != EOF) {
+			   "78DEKLS:X:abcde:fFk:l:n:rxG")) != -1) {
 		switch(ch) {
 		case '8':
 			eight = 3;	/* binary output and input */
@@ -176,7 +211,7 @@ main(int argc, char **argv)
 		    /* sometimes we don't want a mangled display */
 		    char *p;
 		    if((p = getenv("DISPLAY")))
-			env_define("DISPLAY", (unsigned char*)p);
+			env_define((unsigned char*)"DISPLAY", (unsigned char*)p);
 		    break;
 		}
 		case 'E':
@@ -225,43 +260,25 @@ main(int argc, char **argv)
 			set_escape_char(optarg);
 			break;
 		case 'f':
-#if defined(AUTHENTICATION) && defined(KRB5) && defined(FORWARD)
-			if ((forward_flags & OPTS_FORWARD_CREDS) &&
-			    !default_forward) {
-			    fprintf(stderr,
-				    "%s: Only one of -f and -F allowed.\n",
-				    prompt);
-			    usage();
-			}
-			forward_flags |= OPTS_FORWARD_CREDS;
-#else
-			fprintf(stderr,
-			 "%s: Warning: -f ignored, no Kerberos V5 support.\n",
-				prompt);
-#endif
-			break;
 		case 'F':
+		case 'G':
 #if defined(AUTHENTICATION) && defined(KRB5) && defined(FORWARD)
-			if ((forward_flags & OPTS_FORWARD_CREDS) &&
-			    !default_forward) {
+			if (forward_option) {
 			    fprintf(stderr,
-				    "%s: Only one of -f and -F allowed.\n",
+				    "%s: Only one of -f, -F and -G allowed.\n",
 				    prompt);
 			    usage();
 			}
-			forward_flags |= OPTS_FORWARD_CREDS;
-			forward_flags |= OPTS_FORWARDABLE_CREDS;
+			forward_option = ch;
 #else
 			fprintf(stderr,
-			 "%s: Warning: -F ignored, no Kerberos V5 support.\n",
-				prompt);
+			 "%s: Warning: -%c ignored, no Kerberos V5 support.\n",
+				prompt, ch);
 #endif
 			break;
 		case 'k':
 #if defined(AUTHENTICATION) && defined(KRB4)
 		    {
-			extern char *dest_realm, dst_realm_buf[];
-			extern int dst_realm_sz;
 			dest_realm = dst_realm_buf;
 			strlcpy(dest_realm, optarg, dst_realm_sz);
 		    }
@@ -288,6 +305,7 @@ main(int argc, char **argv)
 #ifdef	ENCRYPTION
 			encrypt_auto(1);
 			decrypt_auto(1);
+			wantencryption = 1;
 			EncryptVerbose(1);
 #else
 			fprintf(stderr,
@@ -295,16 +313,6 @@ main(int argc, char **argv)
 								prompt);
 #endif
 			break;
-		case 'G':
-#if defined(AUTHENTICATION) && defined(KRB5) && defined(FORWARD)
-                        forward_flags ^= OPTS_FORWARD_CREDS;
-                        forward_flags ^= OPTS_FORWARDABLE_CREDS;
-#else
-                        fprintf(stderr,
-                         "%s: Warning: -G ignored, no Kerberos V5 support.\n",
-                                prompt);
-#endif
-                        break;
 
 		case '?':
 		default:
@@ -320,6 +328,7 @@ main(int argc, char **argv)
 #if defined(ENCRYPTION)
 		encrypt_auto(1);
 		decrypt_auto(1);
+		wantencryption = -1;
 #endif
 	}
 
