@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 - 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -38,7 +33,7 @@
 
 #include "gen_locl.h"
 
-RCSID("$Id: gen_copy.c,v 1.9 1999/10/10 04:49:56 assar Exp $");
+RCSID("$Id: gen_copy.c,v 1.14 2003/10/03 00:27:36 lha Exp $");
 
 static void
 copy_primitive (const char *typename, const char *from, const char *to)
@@ -59,11 +54,17 @@ copy_type (const char *from, const char *to, const Type *t)
 	       t->symbol->gen_name, from, to);
       break;
   case TInteger:
+  case TUInteger:
+  case TBoolean:
+  case TEnumerated :
       fprintf(codefile, "*(%s) = *(%s);\n", to, from);
       break;
   case TOctetString:
       copy_primitive ("octet_string", from, to);
       break;
+  case TOID:
+      copy_primitive ("oid", from, to);
+      break;
   case TBitString: {
       fprintf(codefile, "*(%s) = *(%s);\n", to, from);
       break;
@@ -125,6 +126,11 @@ copy_type (const char *from, const char *to, const Type *t)
   case TGeneralString:
       copy_primitive ("general_string", from, to);
       break;
+  case TUTF8String:
+      copy_primitive ("utf8string", from, to);
+      break;
+  case TNull:
+      break;
   case TApplication:
       copy_type (from, to, t->subtype);
       break;
