@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997-2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -38,13 +33,14 @@
 
 #include "iprop.h"
 
-RCSID("$Id: replay_log.c,v 1.5 1999/09/24 14:49:56 joda Exp $");
+RCSID("$Id: replay_log.c,v 1.11 2003/11/18 23:19:22 lha Exp $");
 
-static krb5_context context;
-static kadm5_server_context *server_context;
+int start_version = -1;
+int end_version = -1;
 
 static void
-apply_entry(u_int32_t ver,
+apply_entry(kadm5_server_context *server_context,
+	    u_int32_t ver,
 	    time_t timestamp,
 	    enum kadm_ops op,
 	    u_int32_t len,
@@ -52,21 +48,32 @@ apply_entry(u_int32_t ver,
 {
     krb5_error_code ret;
 
+    if((start_version != -1 && ver < start_version) ||
+       (end_version != -1 && ver > end_version)) {
+	/* XXX skip this entry */
+	krb5_storage_seek(sp, len, SEEK_CUR);
+	return;
+    }
     printf ("ver %u... ", ver);
     fflush (stdout);
 
     ret = kadm5_log_replay (server_context,
 			    op, ver, len, sp);
     if (ret)
-	krb5_warn (context, ret, "kadm5_log_replay");
+	krb5_warn (server_context->context, ret, "kadm5_log_replay");
 
     
     printf ("done\n");
 }
 
+static char *config_file;
 int version_flag;
 int help_flag;
+
 struct getargs args[] = {
+    { "config-file", 'c', arg_string, &config_file },
+    { "start-version", 0, arg_integer, &start_version, "start replay with this version" },
+    { "end-version", 0, arg_integer, &end_version, "end replay with this version" },
     { "version", 0, arg_flag, &version_flag },
     { "help", 0, arg_flag, &help_flag }
 };
@@ -75,9 +82,12 @@ int num_args = sizeof(args) / sizeof(args[0]);
 int
 main(int argc, char **argv)
 {
+    krb5_context context;
     krb5_error_code ret;
     void *kadm_handle;
     kadm5_config_params conf;
+    kadm5_server_context *server_context;
+    char **files;
 
     krb5_program_setup(&context, argc, argv, args, num_args, NULL);
     
@@ -88,6 +98,18 @@ main(int argc, char **argv)
 	exit(0);
     }
 
+    if (config_file == NULL)
+	config_file = HDB_DB_DIR "/kdc.conf";
+
+    ret = krb5_prepend_config_files_default(config_file, &files);
+    if (ret)
+	krb5_err(context, 1, ret, "getting configuration files");
+
+    ret = krb5_set_config_files(context, files);
+    krb5_free_config_files(files);
+    if (ret)
+	krb5_err(context, 1, ret, "reading configuration files");
+
     memset(&conf, 0, sizeof(conf));
     ret = kadm5_init_with_password_ctx (context,
 					KADM5_ADMIN_SERVICE,
@@ -100,9 +122,9 @@ main(int argc, char **argv)
 
     server_context = (kadm5_server_context *)kadm_handle;
 
-    ret = server_context->db->open(context,
-				   server_context->db,
-				   O_RDWR | O_CREAT, 0);
+    ret = server_context->db->hdb_open(context,
+				       server_context->db,
+				       O_RDWR | O_CREAT, 0);
     if (ret)
 	krb5_err (context, 1, ret, "db->open");
 
@@ -116,7 +138,7 @@ main(int argc, char **argv)
     ret = kadm5_log_end (server_context);
     if (ret)
 	krb5_warn(context, ret, "kadm5_log_end");
-    ret = server_context->db->close (context, server_context->db);
+    ret = server_context->db->hdb_close (context, server_context->db);
     if (ret)
 	krb5_err (context, 1, ret, "db->close");
     return 0;
