@@ -33,7 +33,7 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: init_creds_pw.c,v 1.85 2005/05/19 07:42:06 lha Exp $");
+RCSID("$Id: init_creds_pw.c,v 1.85.2.1 2005/10/12 12:51:48 lha Exp $");
 
 typedef struct krb5_get_init_creds_ctx {
     krb5_kdc_flags flags;
@@ -274,11 +274,11 @@ get_init_creds_common(krb5_context context,
 	options = &default_opt;
     }
 
-    if (options->private) {
-	ctx->password = options->private->password;
-	ctx->key_proc = options->private->key_proc;
-	ctx->req_pac = options->private->req_pac;
-	ctx->pk_init_ctx = options->private->pk_init_ctx;
+    if (options->opt_private) {
+	ctx->password = options->opt_private->password;
+	ctx->key_proc = options->opt_private->key_proc;
+	ctx->req_pac = options->opt_private->req_pac;
+	ctx->pk_init_ctx = options->opt_private->pk_init_ctx;
     } else
 	ctx->req_pac = KRB5_PA_PAC_DONT_CARE;
 
@@ -1458,8 +1458,8 @@ krb5_get_init_creds_password(krb5_context context,
 	return ret;
 
     if (password == NULL &&
-	options->private->password == NULL &&
-	options->private->pk_init_ctx == NULL)
+	options->opt_private->password == NULL &&
+	options->opt_private->pk_init_ctx == NULL)
     {
 	krb5_prompt prompt;
 	krb5_data password_data;
@@ -1487,7 +1487,7 @@ krb5_get_init_creds_password(krb5_context context,
 	password = password_data.data;
     }
 
-    if (options->private->password == NULL) {
+    if (options->opt_private->password == NULL) {
 	ret = krb5_get_init_creds_opt_set_pa_password(context, options,
 						      password, NULL);
 	if (ret) {
